<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\Support\Arr;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\TypeParser;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\ItemParser;
use SimpleXMLIterator;

class YouplayParser implements ItemParser, TypeParser
{

    public const OBJECT_TYPE = 'x-im/youplay';

    /**
     * @var array
     */
    private $parsedTags = [];

    /**
     * @var LinkParser
     */
    protected $linkParser;

    /**
     * @var string
     */
    private $baseUrl = 'https://delivery.youplay.se';

    public function __construct()
    {
        $this->linkParser = new LinkParser([
            ImageLinkParser::LINK_TYPE => new ImageLinkParser()
        ]);
    }

    public function parse(SimpleXMLIterator $element): array
    {
        $item = new Item($element);
        $item->fill($this->fill($element));
        $item->set('config', $this->extractConfig($item));
        $item->set('video', $this->getVideoData($item));
        $item->set('api_url', $this->createApiUrl($item));
        $item->set('thumb', $this->getThumbData($element));
        return [$item];
    }

    private function fill($element)
    {
        if (property_exists($element, 'data')) {
            $dataIterator = $element->data->children();
            $data = [];
            foreach ($dataIterator as $tag => $item) {
                $data[$tag] = (string)$item;
            }

            return $data;
        }

        return [];
    }

    protected function getThumbData(SimpleXMLIterator $element): array
    {
        $links = Arr::flatten($this->linkParser->parseLinks($element), 1);
        foreach ($links as $link) {
            if ($link instanceof Item && $link->getType() === 'image/jpg') {
                return $this->parseThumb($link);
            }
        }

        return [];
    }

    public function typeToParse(): string
    {
        return static::OBJECT_TYPE;
    }

    private function parseThumb(Item $link): array
    {
        $url = $link->getItemAttribute('url', '');

        return array_replace([
            'width' => 0,
            'height' => 0,
            'url' => $url,
            'format' => pathinfo($url, PATHINFO_EXTENSION)
        ], (array)($link->getChild('data') ?? []));
    }

    private function parseEmbedCode(string $tag): array
    {
        if (array_key_exists($tag, $this->parsedTags)) {
            return $this->parsedTags[$tag];
        }

        $attributes = [];
        $pattern = '#(?(DEFINE)
            (?<name>[a-zA-Z][a-zA-Z0-9-:]*)
            (?<value_double>"[^"]+")
            (?<value_single>\'[^\']+\')
            (?<value_none>[^\s>]+)
            (?<value>((?&value_double)|(?&value_single)|(?&value_none)))
        )
        (?<n>(?&name))(=(?<v>(?&value)))?#xs';
        if (preg_match_all($pattern, $tag, $matches, PREG_SET_ORDER)) {
            foreach ($matches as $match) {
                $attributes[$match['n']] = isset($match['v']) ? trim($match['v'], '\'"') : null;
            }
        }
        $this->parsedTags[$tag] = $attributes;

        return $attributes;
    }

    private function extractConfig(Item $item): array
    {

        if (!isset($item->embedCode)) {
            return [];
        }

        $scriptData = $this->parseEmbedCode($item->embedCode);

        if (array_key_exists('data-config', $scriptData)) {
            return json_decode($scriptData['data-config'], true);
        }

        return [];
    }

    private function getVideoData(Item $item): array
    {
        $partId = str_replace('x-im://youplay/', '', $item->get('uri'));
        $videoFormat = 'mp4';

        return [
            'url' => "{$this->baseUrl}/videos/{$partId }/720?fmt={$videoFormat}",
            'format' => 'mp4'
        ];
    }

    private function createApiUrl(Item $item): string
    {
        $config = $this->extractConfig($item);
        return $config ? "{$this->baseUrl}/api/v2/videodata?zone_id={$config['zone_id']}&part_id={$config['part_id']}" : '';
    }
}
